/* ************************************************************************
 * Copyright 2006 VMware, Inc.  All rights reserved. -- VMware Confidential
 * ************************************************************************/

/* 
 * vmciDatagram.h --
 *
 *      Simple Datagram API for the Linux guest driver.
 */

#ifndef __VMCI_DATAGRAM_H__
#define __VMCI_DATAGRAM_H__

#define INCLUDE_ALLOW_MODULE
#include "includeCheck.h"

#include "vmci_defs.h"
#include "vmci_kernel_defs.h"
#include "vmci_infrastructure.h"
#include "vmci_kernel_defs.h"
#include "circList.h"

struct CallEntry {
   ListItem      listItem; /* For queuing. */
   VMCICall *call;    /* Pending call. */
};

typedef struct VMCIDatagramProcess {
   VMCILock  callQueueLock;
   VMCIHandle       handle;
   VMCIHost  host;
   uint32         pendingCalls;
   ListItem       *callQueue;
} VMCIDatagramProcess;

void VMCIDatagram_Init(void);
Bool VMCIDatagram_CheckHostCapabilities(void);
VMCIHandle VMCIDatagram_CreateHnd(VMCIId resourceID, uint32 flags,
                                VMCIDatagramCB notifyCB, void *notifyData);
void VMCIDatagram_DestroyHnd(VMCIHandle handle);
int VMCIDatagram_Send(VMCIHandle dstHandle, VMCIHandle srcHandle,
                      uint32 payloadSize, void *payload);
int VMCIDatagramProcess_Create(VMCIDatagramProcess **outDgmProc,
                               const int eventHnd);
void VMCIDatagramProcess_Destroy(VMCIDatagramProcess *dgmProc);
int VMCIDatagramProcess_ReadCall(VMCIDatagramProcess *dgmProc, void *buf,
                                 size_t bufSize);
		
#endif //__VMCI_DATAGRAM_H__
