/* **********************************************************
 * Copyright 2006 VMware, Inc.  All rights reserved.
 * -- VMware Confidential
 * **********************************************************/

/*
 * vmciTest.c --
 *
 *      Kernel-level API tests and test drivercall handler.
 */

#ifdef VMX86_DEBUG

#if defined(linux)
#include "driver-config.h"
   
#include <linux/kernel.h>  
#endif

#include "vm_basic_types.h"
#include "vmci_defs.h"
#include "vmci_call_defs.h"
#include "vmci_kernel_defs.h"
#include "vmciTest.h"
#include "vmci_infrastructure.h"
#include "vmciDatagram.h"
#include "vmciResource.h"
#include "vmciSharedMemory.h"
#include "vmciUtil.h"
#include "vmciInt.h"

#define LGPFX "VMCITest: "

static int64 VMCITestDriverHandler(void *ctObj,
                                   VMCIObjType ctObjType,
                                   VMCICall *call);
static int VMCITestGuestCallHandler(VMCIId contextID, void *clientData,
                                    VMCICall *call);


/*
 *-----------------------------------------------------------------------------
 *
 * VMCITest_Init --
 *
 *      Register test drivercall handler.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *
 *-----------------------------------------------------------------------------
 */

void
VMCITest_Init(void)
{
   VMCI_RegisterDriverHandler(VMCI_TEST_CALL, VMCITestDriverHandler);
   VMCI_RegisterHandler(VMCI_TEST_CALL, VMCITestGuestCallHandler, 0, NULL);
}


/*
 *-----------------------------------------------------------------------------
 *
 * VMCITestDriverHandler --
 *
 *      Handler for VMCI_TEST_CALL drivercalls.
 *
 * Results:
 *      VMCI_SUCCESS if successful, appropriate error code otherwise.
 *
 * Side effects:
 *      None.
 *
 *-----------------------------------------------------------------------------
 */

static int64
VMCITestDriverHandler(void *ctObj,         // IN:
                      VMCIObjType ctObjType, // IN:
                      VMCICall *call) // IN:
{
   int result = -1;
   int command;

   if (call->h.size != (sizeof(VMCITestCallHdr))) {
      VMCI_LOG((LGPFX"Test driver call, invalid size (%d != %d)\n",
                call->h.size, (int)sizeof(VMCITestCallHdr)));
      return -1;
   }

   command = ((VMCITestCallHdr*)call)->command;

   switch(command) {
   case VMCI_TEST_INIT: {
      result = VMCI_HyperCall(call);
      break;
   }
   case VMCI_TEST_CLEANUP: {
      result = VMCI_HyperCall(call);
      break;
   }
   case VMCI_TEST_GUESTCALL: {
      /* XXX Guestcall tests not yet implemented. */
      result = VMCI_SUCCESS;
      break;
   }
   default:
      VMCI_LOG((LGPFX"Unknown command: %d\n", command));
      result = -1;
      break;
   }

   return result;

   return 0;
}


/*
 *-----------------------------------------------------------------------------
 *
 * VMCITestGuestCallHandler --
 *
 *      Handles test guest calls.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *
 *-----------------------------------------------------------------------------
 */

static int 
VMCITestGuestCallHandler(VMCIId contextID, // IN 
                         void *clientData,      // IN
                         VMCICall *call)   // IN
{
   VMCI_LOG((LGPFX"GCHandler: Received guest call with vector %d and "
             "size %d.\n", call->h.vector, call->h.size));
   return 10;
}

#endif // VMX86_DEBUG
